@@ -13,4 +13,4 @@
 ## usethis namespace: end
 NULL
 
-utils::globalVariables(c("attribute", "name", "simba_config_dirs"))
+utils::globalVariables(c("attribute", "name"))
